# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: node.rb 82 2008-09-25 13:06:43Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

class Node
  @@nodes = Hash.new

  def initialize(name)
    @name = name
    @status = Hash.new
    @nagios = Hash.new
    @nstat = [nil, "ok", nil]

    @@nodes[@name] = self
    @vnodes = Array.new

    # Read nagios status
    status = Client.getTS.read_all([:NagiosStatus, @name, nil, nil])
    status.each{ |s|
	setNagiosStatus(s[2], s[3])
    }

    updStatus
  end

  # Assigns a +VNode+ to a +Node+ instance.
  def add(vnode)
    @vnodes.push(vnode)
    vnode.refreshStatus(@nstat)
    updateTooltip
  end

  # Add the given +VNode+ to a named +Node+ instance.
  def self.add(name, vnode)
    if @@nodes[name] then
	node = @@nodes[name]
    else
	node = Node.new(name)
    end

    node.add(vnode)

    return node
  end

  def self.getNode(name)
    @@nodes[name]
  end

  def updateTooltip
    tt = "<b>#{@name}</b>\n"

    ttmax = 0
    @status.each_pair{|c, s|
	chart = Chart.getChart([@name, c])
	ttmax = chart.opts['title'].length if chart && ttmax < chart.opts['title'].length
    }
    @nagios.each_pair{|c, s|
	ttmax = c.length if ttmax < c.length
    }

    @status.sort.each{|c, s|
	chart = Chart.getChart([@name, c])

	if chart then
	    tt += "\n<tt>" + sprintf("%#{ttmax}s: ", chart.opts['title'])
	    tt += "<b><span foreground='" + Status.text_color(s[1]) + "'>"
	    if s[2] then
		tt += s[2] + "</span></b></tt>"
	    else
		tt += s[1].upcase + "</span></b></tt>"
	    end
	else
	    tt += "\n<tt>" + sprintf("%#{ttmax}s: ", c)
	    tt += "<b><span foreground='" + Status.text_color(s[1]) + "'>"
	    if s[2] then
		tt += s[2] + "</span></b></tt>"
	    else
		tt += s[1].upcase + "</span></b></tt>"
	    end
	end
    }
    if @nagios.size > 0 then
	tt += "\n" if @status.size > 0
	@nagios.sort.each{|c, s|
	    tt += "\n<tt>" + sprintf("%#{ttmax}s: ", c)
	    tt += "<b><span foreground='" + Status.text_color(s[1]) + "'>"
	    if s[2] then
		tt += s[2] + "</span></b></tt>"
	    else
		tt += s[1].upcase + "</span></b></tt>"
	    end
	}
    end

    @vnodes.each{|vnode|
	vnode.setTooltip(tt)
    }
  end

  def setStatus(sensor=nil, status=nil)
    if sensor != nil and status != nil then
	@status[sensor] = status
    end

    updStatus
  end

  def setNagiosStatus(sensor, status)
    @nagios[sensor] = status

    updStatus
  end

  def updStatus
    nstat = [0, "ok", nil]
    @status.each_pair{|c, s|
	nstat = s if Status.int(s[1]) > Status.int(nstat[1])
    }
    @nagios.each_pair{|c, s|
	nstat = s if Status.int(s[1]) > Status.int(nstat[1])
    }

    if nstat != @nstat then
	@nstat = nstat
    end

    @vnodes.each{|vnode|
        vnode.refreshStatus(@nstat)
    }
    updateTooltip
  end
end

# This class represents a node object in a view.
class VNode
  attr_reader :x, :y

  def initialize(view, node, layout)
    @view = view
    @nstat = [nil, nil, nil]

    @name = node.attributes["name"]
    @sx = @x = node.attributes["x"].to_i
    @sy = @y = node.attributes["y"].to_i

    @vb = Gtk::VBox.new(false, 2)
    if node.attributes['icon'] == nil then
	@icon = nil
    else
	@icon = File.basename(node.attributes['icon'].downcase)
    end
    @img = Gtk::Image.new

    @ebi = Gtk::EventBox.new
    @ebi.add(@img)

    @vb.pack_start(@ebi)

    @eb = Gtk::EventBox.new

    @label = Gtk::Label.new
    @label.use_markup = true
    @label.markup = "<small>#{@name}</small>"
    @eb.add(@label)
    @vb.pack_start(@eb)
    layout.put(@vb, @x, @y)
    @vb.signal_connect("size_allocate") { |widget, alloc|
      nx = @x - @vb.size_request[0]/2
      ny = @y - @img.size_request[1]/2

      if @sx != nx or
          @sy != ny then
        @sx = nx
        @sy = ny
        layout.move(widget, nx, ny)
      end
    }

    @vb.show_all

    Node.add(@name, self)
  end

  def setTooltip(tt)
    begin
	# Older GTK bindings lack tooltip_markup property
	@vb.tooltip_markup = tt.chomp
    rescue
	# Gtk::VBox has now window, use the child's Gtk::EventBox widgets directly
	Client.getTooltips.set_tip(@ebii, tt.gsub(/<[^>]+>/, "").chomp, nil)
	Client.getTooltips.set_tip(@eb, tt.gsub(/<[^>]+>/, "").chomp, nil)
    end
  end

  def refreshStatus(nstat)
    return if nstat == @nstat

    @nstat = nstat

    @eb.modify_bg(Gtk::STATE_NORMAL, Gdk::Color.parse(Status.text_color(@nstat[1])))

    if @icon != nil then
	fn = @icon + Status.icon_color(@nstat[1])
    else
	fn = "geng"
    end

    if File.readable?("res/#{fn}.png") then
	@img.set("res/#{fn}.png")
    else
	@img.set("res/geng.png")
    end

    @view.setStatus("n:#{@name}", @nstat[1])
  end
end
